# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2011 ASPerience SARL (<http://www.asperience.fr>). All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields,osv
import openerp.netsvc

import time
import datetime
from mx import DateTime
from openerp.tools.translate import _
from tools import ustr

class mrp_workcenter_technology(osv.osv):
    _name = 'mrp.workcenter.technology'
    _columns = {
        'name': fields.char('Description', size=256, required=True),
    }
    _defaults = {
    }
mrp_workcenter_technology()

class mrp_workcenter(osv.osv):
    _inherit = 'mrp.workcenter'
    
    def _get_type(self, cr, uid, ids, field_name, arg, context={}):
        res = {}
        for id in ids:
            res[id] = {'type_re': '', 'type_pc': '', 'type_cc': ''}
        for workcenter in self.browse(cr, uid, ids):
            if workcenter.type_workcenter == 're':
                res[id]['type_re'] = workcenter.type
            elif workcenter.type_workcenter == 'pc':
                res[id]['type_pc'] = workcenter.type
            elif workcenter.type_workcenter == 'cc':
                res[id]['type_cc'] = workcenter.type
        return res
    
    def _set_type(self, cr, uid, ids, name, value, arg, context):
        if not value:
            return False
        if isinstance(ids, (int, long)):
            ids = [ids]
        for workcenter in self.browse(cr, uid, ids, context):
            self.write(cr, uid, ids, {'type': value})
        return True
    
    def onchange_type(self, cr, uid, id, type=''):
        res = {'value':{'type': type}}
        return res
    
    def onchange_type_workcenter(self, cr, uid, id, type_workcenter):
        res = {'value':{'type_re': '','type_pc': '', 'type_cc': '', 'type': ''}}
        if type_workcenter == 're' or type_workcenter == 'pc':
            res['value']['type_re'] = 'machine'
        elif type_workcenter == 'cc':
            res['value']['type_re'] = 'machine_hr'
        res['value']['type'] = res['value']['type_re']
        return res
        
    _columns = {
        'type': fields.selection([('machine','Machine'),('hr','Human Resource'),('tool','Tool'),('machine_hr','Machine and HR'),('line','Line'),('energy','Energy')], 'Type', required=True),
        'type_re': fields.function(_get_type, fnct_inv=_set_type, selection=[('machine',_('Machine')),('hr',_('Human Resource')),('tool',_('Tool')),('energy',_('Energy'))],
                                   multi="type",method=True, store=False, type='selection', string='Type'),
        'type_pc': fields.function(_get_type, fnct_inv=_set_type, selection=[('machine',_('Machine')),('hr',_('Human Resource')),('machine_hr',_('Machine and HR')),('line',_('Line'))],
                                   multi="type",method=True, store=False, type='selection', string='Type'),
        'type_cc': fields.function(_get_type, fnct_inv=_set_type, selection=[('machine_hr',_('Machine and HR')),('line',_('Line'))],
                                   multi="type",method=True, store=False, type='selection', string='Type'),
        'type_workcenter': fields.selection([('re',_('Elementary resource')),('pc',_('Work center')),('cc',_('Work line'))], 'Type workcenter', required=True, size=2),
        'parent_id': fields.many2one('mrp.workcenter', "Parent"),
        'child_ids': fields.one2many('mrp.workcenter', 'parent_id','Childs'),
        'technology': fields.many2one('mrp.workcenter.technology', "Technology"),
        'external': fields.boolean('External workcenter'),
        'fixed_capacity': fields.boolean('Fixed capacity'),
        'capacity' : fields.float('Capacity'),
        'critical': fields.boolean('Critical'),
        'property_ids': fields.many2many('mrp.property', 'mrp_workcenter_property_rel','workcenter_id', 'property_id', 'Properties'),
        'warehouse_id': fields.many2one('stock.warehouse', 'Sector', required=True),
        'finite_capacity': fields.boolean('Finite Capacity'),
        'finite_capacity_qty' : fields.float('Finite Capacity Qty'),
        'scheduling': fields.selection([('machine','Machine'),('hr','Human Resource'),('machine_hr','Machine and HR'),('pg','the greatest')], 'Scheduling'),
        }
    _defaults = {
        'time_start': lambda *a: 0,
        'time_stop': lambda *a: 0,
        'time_cycle': lambda *a: 0,
        'type_workcenter': lambda *a: "pc",
    }
    
    def name_search(self, cr, uid, name, args=None, operator='ilike', context=None, limit=80):
        if args:
            if args[0][0] == 'type':
                for val in args[0][2]:
                    if val: 
                        if val in ['line', 'machine_hr']:
                            args = [('type', '=', 'line')]
                            break
                        elif val == 'machine':
                            args = [('type', 'in', ['line','machine_hr','machine'])]
                            break
                        elif val in ['energy', 'hr', 'tool']:
                            args = [('type', 'in', ['line','machine_hr','machine','hr'])]
                            break

        return super(mrp_workcenter, self).name_search(cr, uid, name, args, operator, context, limit)
    
mrp_workcenter()

class mrp_bom_component_mode(osv.osv):
    _name = 'mrp.bom.component.mode'
    _description = 'mrp.bom.component.mode'
    _columns = {
        'name': fields.char('Description', size=256, required=True),
        'dx': fields.text('Dx'),
        'dy': fields.text('Dy'),
        'dz': fields.text('Dz'),
        'dd': fields.text('Dd'),
    }
    _defaults = {
    }
    
mrp_bom_component_mode()

class mrp_bom_component_rel(osv.osv):
    _name = 'mrp.bom.component.rel'
    _description = 'mrp.bom.component.rel'
    _columns = {
        'bom_id': fields.many2one('mrp.bom', 'Bom', required=True, ondelete='cascade'),
        'component_id': fields.many2one('mrp.bom', 'Component', required=True, ondelete='cascade'),
#        'product_component_id': fields.related('component_id','product_id',type='many2one', relation="product.product", string="Product"),
        'qty': fields.float('Quantity', required=True),
        'mode': fields.many2one('mrp.bom.component.mode', 'Mode'),
    }

    def compute_size(self, cr, uid, ids, context={}):
        logger = netsvc.Logger()
        if not ids :
            return False
        result = {}
        for i in self.browse(cr,uid,ids,context):
            space = {}
            space["x"] = context.get("size_x",0.0)
            space["y"] = context.get("size_y",0.0)
            space["z"] = context.get("size_z",0.0)
            space["d"] = context.get("diameter",0.0)
            space["Dx"] = context.get("size_x_specific_delta",0.0)
            space["Dy"] = context.get("size_y_specific_delta",0.0)
            space["Dz"] = context.get("size_z_specific_delta",0.0)
            space["Dd"] = context.get("diameter_specific_delta",0.0)
            
            
            result[i.id] = {
                            "size_x_specific_delta":0.0,
                            "size_y_specific_delta":0.0,
                            "size_z_specific_delta":0.0,
                            "diameter_specific_delta":0.0,
                            "size_x_specific":context.get("size_x",0.0),
                            "size_y_specific":context.get("size_y",0.0),
                            "size_z_specific":context.get("size_z",0.0),
                            "diameter_specific":context.get("diameter",0.0),
            }
            
            try:
                result[i.id]["size_x_specific_delta"] = eval(i.mode.dx,space)
                result[i.id]["size_x_specific"] = result[i.id]["size_x_specific"] - result[i.id]["size_x_specific_delta"]
            except Exception, e:
                logger.notifyChannel('eval size', netsvc.LOG_WARNING, e)
                logger.notifyChannel('eval size', netsvc.LOG_WARNING, "transform x "+str(i.mode.dx))
            try:
                result[i.id]["size_y_specific_delta"] = eval(i.mode.dy,space)
                result[i.id]["size_y_specific"] = result[i.id]["size_y_specific"] - result[i.id]["size_y_specific_delta"]
            except Exception, e:
                logger.notifyChannel('eval size', netsvc.LOG_WARNING, e)
                logger.notifyChannel('eval size', netsvc.LOG_WARNING, "transform y "+str(i.mode.dy))
            try:
                result[i.id]["size_z_specific_delta"] = eval(i.mode.dz,space)
                result[i.id]["size_z_specific"] = result[i.id]["size_z_specific"] - result[i.id]["size_z_specific_delta"]
            except Exception, e:
                logger.notifyChannel('eval size', netsvc.LOG_WARNING, e)
                logger.notifyChannel('eval size', netsvc.LOG_WARNING, "transform z "+str(i.mode.dz))
            try:
                result[i.id]["diameter_specific_delta"] = eval(i.mode.dd,space)
                result[i.id]["diameter_specific"] = result[i.id]["diameter_specific"] - result[i.id]["diameter_specific_delta"]
            except Exception, e:
                logger.notifyChannel('eval size', netsvc.LOG_WARNING, e) 
                logger.notifyChannel('eval size', netsvc.LOG_WARNING, "transform d "+str(i.mode.dd))
            
        return result
    
    _defaults = {
    }
mrp_bom_component_rel()

def rounding(f, r):
    if not r:
        return f
    return round(f / r) * r

class mrp_bom(osv.osv):
    _inherit = 'mrp.bom'
    
#    def __init__(self, pool, cr):
#        super(mrp_bom,self).__init__(pool,cr)
#        cr.execute('select count(*) from mrp_bom_childs_rel')
#        nb1 = cr.dictfetchall()[0]['count']
#        cr.execute('select count(*) from mrp_bom_component_rel')
#        nb2 = cr.dictfetchall()[0]['count']
#        if nb1 and not nb2 :
#            cr.execute('insert into mrp_bom_component_rel\
#                        (bom_id,component_id,qty)\
#                        select a.bom_id, a.child_id, b.product_qty\
#                        from mrp_bom_childs_rel a join mrp_bom b on a.bom_id = b.id')
#            
#            map_mode = {
#                        'bottom':['.DES.'],
#                        'top':['PLATEAU'],
#                        'back':['.DER.'],
#                        'front':['.FACE.'],
#                        'side':['.COT.'],
#                        }
#            for i in map_mode:
#                for j in map_mode[i]:
#                    cr.execute("update mrp_bom_component_rel\
#                                set mode = (    select id\
#                                        from mrp_bom_component_mode\
#                                        where name = '"+ustr(i)+"')\
#                                where id in (\
#                                    select a.id\
#                                    from mrp_bom_component_rel a join mrp_bom b on a.component_id = b.id join product_product c on b.product_id = c.id join product_template d on c.product_tmpl_id = d.id\
#                                    where c.default_code like '%"+ustr(j)+"%');")
            
            
    def _child_compute(self, cr, uid, ids, name, arg, context={}):
        result = {}
        for bom in self.browse(cr, uid, ids, context=context):
            result[bom.id] = map(lambda x: x.component_id.id, bom.bom_components)
            if bom.bom_components:
                continue
            ok = ((name=='child_complete_ids') and (bom.product_id.supply_method=='produce'))
            if bom.type=='phantom' or ok:
                sids = self.pool.get('mrp.bom').search(cr, uid, [('bom_id','=',False),('product_id','=',bom.product_id.id)])
                if sids:
                    bom2 = self.pool.get('mrp.bom').browse(cr, uid, sids[0], context=context)
                    result[bom.id] += map(lambda x: x.component_id.id, bom2.bom_components)
        return result

    def _compute_type(self, cr, uid, ids, field_name, arg, context):
        res = dict(map(lambda x: (x,''), ids))
        for line in self.browse(cr, uid, ids):
            if line.type=='phantom' and not line.bom_id:
                res[line.id] = 'set'
                continue
            if line.bom_lines or line.bom_components or line.type=='phantom':
                continue
            if line.product_id.supply_method=='produce':
                if line.product_id.procure_method=='make_to_stock':
                    res[line.id] = 'stock'
                else:
                    res[line.id] = 'order'
        return res

    _columns = {
            'bom_type': fields.selection([('ass','Assemble'),('exp','Disassemble'),('cut','Cutting'),('cutinput','Cutting Input'),('picking','Picking'),('delay','Delay'),('control','Quality control')], 'BOM Type', required=True ),
            'bom_lines_multi': fields.many2many('mrp.bom', 'mrp_bom_childs_rel', 'bom_id', 'child_id', 'Child BoMs'),
            'bom_parent_multi': fields.many2many('mrp.bom', 'mrp_bom_childs_rel', 'child_id', 'bom_id', 'Parent BoMs'),
            'bom_components': fields.one2many('mrp.bom.component.rel', 'bom_id', 'Components'),
            'bom_components_parent': fields.one2many('mrp.bom.component.rel', 'component_id', 'Parent Components'),
            'method': fields.function(_compute_type, string='Method', method=True, type='selection', selection=[('',''),('stock','On Stock'),('order','On Order'),('set','Set / Pack')]),
            'type_group_ids': fields.many2many('product.type.property.option', 'mrp_bom_options_rel', 'bom_id', 'type_id', 'GOO',help="Used to map which property option is taken from previous product in assembly chain, or pushed to in dissambly chain"),
            'child_complete_ids': fields.function(_child_compute,relation='mrp.bom', method=True, string="BoM Hyerarchy", type='many2many'),
#            'group_property_ids': fields.many2many('mrp.property.group.option', 'mrp_bom_group_property_rel','bom_id', 'group_id', 'BOM Property Group Rel', help="Used to produce with properties not defined in sale order"),
            'sub_product_ids': fields.many2many('mrp.subproduct', 'mrp_bom_subproduct_rel','bom_id', 'sub_id', 'Sub products'),
    }
    _defaults = {
            'bom_type': lambda *a: 'ass'
    }
    
mrp_bom()

class mrp_production(osv.osv):
    _inherit = 'mrp.production'
    
    def _size(self, cr, uid, ids, field_name, arg, context):
       res = {}
       for proc in self.browse(cr, uid, ids):
           res[proc.id] = {'size_x': 0.0, 'size_y': 0.0, 'size_z': 0.0 , 'diameter': 0.0}
           if proc.product_id:
                res[proc.id]['size_x'] = proc.product_id.size_x or proc.product_id.product_tmpl_id.size_x_tmpl or 0.0
                res[proc.id]['size_y'] = proc.product_id.size_y or proc.product_id.product_tmpl_id.size_y_tmpl or 0.0
                res[proc.id]['size_z'] = proc.product_id.size_z or proc.product_id.product_tmpl_id.size_z_tmpl or 0.0
                res[proc.id]['diameter'] = proc.product_id.diameter or proc.product_id.product_tmpl_id.diameter_tmpl or 0.0
       return res
   
    _columns = {
        'procurement_ids': fields.one2many('procurement.order', 'production_id', 'Procurements'),
        'specific_production':fields.boolean('Specific Production'),
        'note_production':fields.text('Production Note'),
        'size_x': fields.function(_size, multi="size", method=True, string='Width', digits=(16, 5), type='float'),
        'size_y': fields.function(_size, multi="size", method=True, string='Length', digits=(16, 5), type='float'),
        'size_z': fields.function(_size, multi="size", method=True, string='Thickness', digits=(16, 5), type='float'),
        'diameter': fields.function(_size, multi="size", method=True, string='Diameter', digits=(16, 5), type='float'),
        'size_x_specific': fields.float('Width specific', digits=(16, 5), readonly=True),
        'size_y_specific': fields.float('Length specific', digits=(16, 5), readonly=True),
        'size_z_specific': fields.float('Thickness specific', digits=(16, 5), readonly=True),
        'diameter_specific': fields.float('Diameter specific', digits=(16, 5), readonly=True),
        'size_x_specific_delta': fields.float('Width specific delta', digits=(16, 5), readonly=True),
        'size_y_specific_delta': fields.float('Length specific delta', digits=(16, 5), readonly=True),
        'size_z_specific_delta': fields.float('Thickness specific delta', digits=(16, 5), readonly=True),
        'diameter_specific_delta': fields.float('Diameter specific delta', digits=(16, 5), readonly=True),
        'uom_s_size_specific' : fields.many2one('product.uom', 'Size Uom', help="Default united of measure used for operations size"),
    }
    _order = 'id desc'
    
    def action_production_end(self, cr, uid, ids):
#        move_ids = []
        for production in self.browse(cr, uid, ids):
            for res in production.move_lines:
                for move in production.move_created_ids:
                    #XXX must use the orm
#                    cr.execute('INSERT INTO stock_move_history_ids \
#                            (parent_id, child_id) VALUES (%s,%s)',
#                            (res.id, move.id))
                    #ASPerience: test des valeurs pour éviter les erreurs de réécriture
                    cr.execute('select parent_id,child_id from stock_move_history_ids where parent_id=%s and child_id=%s', (res.id, move.id))
                    if len(cr.fetchall()) == 0:
                        cr.execute('insert into stock_move_history_ids (parent_id,child_id) values (%s,%s)', (res.id, move.id))
                    else:
                        logger = netsvc.Logger()
                        logger.notifyChannel("warning", netsvc.LOG_WARNING," ERROR insert into stock_move_history_ids parent_id=%s and child_id=%s'." % (str(move.id), str(move.move_dest_id.id)))
                    #Fin ASPerience
#                move_ids.append(res.id)
            vals= {'state':'confirmed'}
            new_moves = [x.id for x in production.move_created_ids]
            self.pool.get('stock.move').write(cr, uid, new_moves, vals)
            if not production.date_finnished:
                self.write(cr, uid, [production.id],
                        {'date_finnished': time.strftime('%Y-%m-%d %H:%M:%S')})
            self.pool.get('stock.move').check_assign(cr, uid, new_moves)
            self.pool.get('stock.move').action_done(cr, uid, new_moves)
            self._costs_generate(cr, uid, production)
#        self.pool.get('stock.move').action_done(cr, uid, move_ids)
        self.write(cr,  uid, ids, {'state': 'done'})
        return True
    
mrp_production()
    
class mrp_production_product_line(osv.osv):
    _inherit = 'mrp.production.product.line'
    
    _columns = {
        'property_ids': fields.many2many('mrp.property', 'mrp_production_product_line_property_rel', 'line_id','property_id', 'Properties'),
        'group_property_ids': fields.many2many('mrp.property.group.option', 'mrp_production_line_group_property_rel','production_line_id', 'group_id', 'Product Property Group Rel '),
        'procurement_id': fields.many2one('procurement.order', 'Procurement'),
        'option_ids': fields.many2many('product.links', 'mrp_production_product_line_options_rel','line_id', 'links_id', 'Option'),
        'bom_id': fields.many2one('mrp.bom', 'Bom'),
        'specific_production':fields.boolean('Specific Production'),
        'note_production':fields.text('Production Note'),
        'size_x_specific': fields.float('Width specific', digits=(16, 5), readonly=True ),
        'size_y_specific': fields.float('Length specific', digits=(16, 5), readonly=True),
        'size_z_specific': fields.float('Thickness specific', digits=(16, 5), readonly=True),
        'diameter_specific': fields.float('Diameter specific', digits=(16, 5), readonly=True),
        'size_x_specific_delta': fields.float('Width specific delta', digits=(16, 5), readonly=True),
        'size_y_specific_delta': fields.float('Length specific delta', digits=(16, 5), readonly=True),
        'size_z_specific_delta': fields.float('Thickness specific delta', digits=(16, 5), readonly=True),
        'diameter_specific_delta': fields.float('Diameter specific delta', digits=(16, 5), readonly=True),
        'uom_s_size_specific' : fields.many2one('product.uom', 'Size Uom', help="Default united of measure used for operations size"),
    }
mrp_production_product_line()

class stock_production_lot(osv.osv):
    _inherit = 'stock.production.lot'
    
    def search_specific(self,cr,uid,vals,delay):
        filter = [('product_id','=',vals['product_id'])]
        filter_key = ['density','diameter','shape','size_x','size_y','size_z','weight']
        for key in filter_key:
            if vals[key]:
                filter.append((key,'=',vals[key]))
                        
        today=datetime.datetime.today()
        if delay:
            limit = (today-datetime.timedelta(days=delay)).strftime('%Y-%m-%d %H:%M:%S')
            filter.append(('date','>',limit))
        
        result = self.search(cr,uid,filter)
        set_property_ids = set(vals['property_ids'][0][2])
        set_group_property_ids = set(vals['group_property_ids'][0][2])
        set_option_ids = set(vals['option_ids'][0][2])
        for lot_id in result:
            lot = self.browse(cr,uid,lot_id)
            lot_property_ids = set([i.id for i in lot.property_ids])
            lot_group_property_ids = set([i.id for i in lot.group_property_ids])
            lot_option_ids = set([i.id for i in lot.option_ids])
            if lot_property_ids == set_property_ids and lot_group_property_ids == set_group_property_ids and lot_option_ids == set_option_ids :
                return lot.id
        return False
        
    _columns = {
        'property_ids': fields.many2many('mrp.property', 'mrp_production_lot_property_rel', 'lot_id','property_id', 'Properties'),
        'group_property_ids': fields.many2many('mrp.property.group.option', 'mrp_production_lot_group_property_rel','production_lot_id', 'group_id', 'Product Property Group Rel '),
        'option_ids': fields.many2many('product.links', 'mrp_production_lot_options_rel','production_lot_id', 'links_id', 'Options'),
    }

    _defaults = {
    }
stock_production_lot()

class mrp_routing(osv.osv):
    _inherit = 'mrp.routing'
    _columns = {
        'technical_data': fields.char('Technical Data', size=128),
    }
    _defaults = {
        'active': lambda *a: 1,
    }
mrp_routing()

class mrp_routing_workcenter_subproduct(osv.osv):
    _name = 'mrp.routing.workcenter.subproduct'
    _description = 'mrp.routing.workcenter.subproduct'
    _columns = {
        'name': fields.float('Percent', required=True),
        'subproduct_id': fields.many2one('mrp.subproduct', 'Subproduct', required=True),
        'workcenter_line_id': fields.many2one('mrp.routing.workcenter', 'Routing Workcenter', required=True),
    }
mrp_routing_workcenter_subproduct()

class mrp_routing_workcenter(osv.osv):
    _inherit = 'mrp.routing.workcenter'
    _columns = {
        'sub_ids': fields.one2many('mrp.routing.workcenter.subproduct', 'workcenter_line_id', 'Sub Product'),
        'bom_ids': fields.many2many('mrp.bom', 'routing_workcenter_mrp_bom_rel', 'workcenter_line_id', 'bom_id', 'Components'),
    }
    _defaults = {
    }
mrp_routing_workcenter()

class mrp_production_workcenter_line(osv.osv):
    _inherit = 'mrp.production.workcenter.line'
    _columns = {
        'move_ids' : fields.many2many('stock.move', 'mrp_production_workcenter_line_stock_move_rel', 'line_id', 'move_id', 'Moves'),
    }
    _defaults = {
    }
mrp_production_workcenter_line()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: