# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2007-2014 ASPerience SARL (<http://www.asperience.fr>).
#    All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from openerp.osv import fields,osv
from openerp.tools.translate import _
#import openerp.netsvc

class mrp_workcenter_technology(osv.osv):
    _name = 'mrp.workcenter.technology'
    _columns = {
        'name': fields.char('Description', size=256, required=True),
    }
    _defaults = {
    }
mrp_workcenter_technology()

class mrp_workcenter(osv.osv):
    _inherit = 'mrp.workcenter'
    
    def _get_type(self, cr, uid, ids, field_name, arg, context={}):
        res = {}
        for id in ids:
            res[id] = {'type_re': '', 'type_pc': '', 'type_cc': ''}
        for workcenter in self.browse(cr, uid, ids):
            if workcenter.type_workcenter == 're':
                res[id]['type_re'] = workcenter.type
            elif workcenter.type_workcenter == 'pc':
                res[id]['type_pc'] = workcenter.type
            elif workcenter.type_workcenter == 'cc':
                res[id]['type_cc'] = workcenter.type
        return res
    
    def _set_type(self, cr, uid, ids, name, value, arg, context):
        if not value:
            return False
        if isinstance(ids, (int, long)):
            ids = [ids]
        for workcenter in self.browse(cr, uid, ids, context):
            self.write(cr, uid, ids, {'type': value})
        return True
    
    def onchange_type(self, cr, uid, id, type=''):
        res = {'value':{'type': type}}
        return res
    
    def onchange_type_workcenter(self, cr, uid, id, type_workcenter):
        res = {'value':{'type_re': '','type_pc': '', 'type_cc': '', 'type': ''}}
        if type_workcenter == 're' or type_workcenter == 'pc':
            res['value']['type_re'] = 'machine'
        elif type_workcenter == 'cc':
            res['value']['type_re'] = 'machine_hr'
        res['value']['type'] = res['value']['type_re']
        return res
        
    _columns = {
        'sequence': fields.integer('Sequence'),
        'type': fields.selection([('machine','Machine'),('hr','Human Resource'),('tool','Tool'),('machine_hr','Machine and HR'),('line','Line'),('energy','Energy')], 'Type', required=True),
        'type_re': fields.function(_get_type, fnct_inv=_set_type, selection=[('machine',_('Machine')),('hr',_('Human Resource')),('tool',_('Tool')),('energy',_('Energy'))],
                                   multi="type",method=True, store=False, type='selection', string='Type'),
        'type_pc': fields.function(_get_type, fnct_inv=_set_type, selection=[('machine',_('Machine')),('hr',_('Human Resource')),('machine_hr',_('Machine and HR')),('line',_('Line'))],
                                   multi="type",method=True, store=False, type='selection', string='Type'),
        'type_cc': fields.function(_get_type, fnct_inv=_set_type, selection=[('machine_hr',_('Machine and HR')),('line',_('Line'))],
                                   multi="type",method=True, store=False, type='selection', string='Type'),
        'type_workcenter': fields.selection([('re',_('Elementary resource')),('pc',_('Work center')),('cc',_('Work line'))], 'Type workcenter', required=True, size=2),
        'scheduling': fields.selection([('machine','Machine'),('hr','Human Resource'),('machine_hr','Machine and HR'),('pg','The greatest')], 'Scheduling'),
        'parent_id': fields.many2one('mrp.workcenter', 'Parent'),
        'child_ids': fields.one2many('mrp.workcenter', 'parent_id','Childs'),
        'technology': fields.many2one('mrp.workcenter.technology', 'Technology'),
        'warehouse_id': fields.many2one('stock.warehouse', 'Sector', required=True),
        'property_ids': fields.many2many('mrp.property', 'mrp_workcenter_property_rel','workcenter_id', 'property_id', 'Properties'),
        #A redéfinir
        'external': fields.boolean('External workcenter'),
        'fixed_capacity': fields.boolean('Fixed capacity'),
        #'capacity_qty' : fields.float('Capacity'),
        'critical': fields.boolean('Critical'),
        'finite_capacity': fields.boolean('Finite Capacity'),
        'finite_capacity_qty' : fields.float('Finite Capacity Qty'),
        }
    _defaults = {
        'sequence': lambda *a: 0,
        'time_start': lambda *a: 0,
        'time_stop': lambda *a: 0,
        'time_cycle': lambda *a: 0,
        'type_workcenter': lambda *a: 'pc',
    }
    
    def name_search(self, cr, uid, name, args=None, operator='ilike', context=None, limit=80):
        print "name:",name
        print "args:",args
        print "operator:",operator
        print "context:",context
        print "limit:",limit
        if args:
            if args[0][0] == 'type':
                for val in args[0][2]:
                    if val: 
                        if val in ['line', 'machine_hr']:
                            args = [('type', '=', 'line')]
                            break
                        elif val == 'machine':
                            args = [('type', 'in', ['line','machine_hr','machine'])]
                            break
                        elif val in ['energy', 'hr', 'tool']:
                            args = [('type', 'in', ['line','machine_hr','machine','hr'])]
                            break

        print "args:",args
        return super(mrp_workcenter, self).name_search(cr, uid, name, args=args, operator=operator, context=context, limit=limit)
    
mrp_workcenter()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: