# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2007-2014 ASPerience SARL (<http://www.asperience.fr>).
#    All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from openerp.osv import fields,osv
#from openerp.tools.translate import _
#import datetime

class stock_production_lot(osv.osv):
    _inherit = 'stock.production.lot'
    
    def search_specific(self,cr,uid,vals,delay):
        filter = [('product_id','=',vals['product_id'])]
        filter_key = ['density','diameter','shape','size_x','size_y','size_z','weight']
        for key in filter_key:
            if vals[key]:
                filter.append((key,'=',vals[key]))
                        
        today=datetime.datetime.today()
        if delay:
            limit = (today-datetime.timedelta(days=delay)).strftime('%Y-%m-%d %H:%M:%S')
            filter.append(('date','>',limit))
        
        result = self.search(cr,uid,filter)
        set_property_ids = set(vals['property_ids'][0][2])
        set_group_property_ids = set(vals['group_property_ids'][0][2])
        set_option_ids = set(vals['option_ids'][0][2])
        for lot_id in result:
            lot = self.browse(cr,uid,lot_id)
            lot_property_ids = set([i.id for i in lot.property_ids])
            lot_group_property_ids = set([i.id for i in lot.group_property_ids])
            lot_option_ids = set([i.id for i in lot.option_ids])
            if lot_property_ids == set_property_ids and lot_group_property_ids == set_group_property_ids and lot_option_ids == set_option_ids :
                return lot.id
        return False
        
    _columns = {
        #Dimensions
        'size_x': fields.float('Width', digits=(16, 5)),
        'size_y': fields.float('Length', digits=(16, 5)),
        'size_z': fields.float('Thickness', digits=(16, 5)),
        'density': fields.float('Density', digits=(16, 5)),
        'shape' : fields.selection([('quadrangular', 'Quadrangular'), ('cylindrical', 'Cylindrical'), ('other', 'Other')], 'Shape', required=True),
        'diameter' : fields.float('Diameter', digits=(16, 5)),
        'weight': fields.float('Weight', digits=(16, 5)),
        'uom_d_size' : fields.related('product_id', 'uom_d_size', help="Default united of measure density used for operations size to cube"),
        'uom_d_weight': fields.related('product_id', 'uom_d_weight', help="Default united of measure density used for operations weight"),
        'uom_s_size' : fields.related('product_id', 'uom_s_size', help="Default united of measure used for operations size"),

        #Properties
        'property_ids': fields.many2many('mrp.property', 'mrp_production_lot_property_rel', 'lot_id','property_id', 'Properties'),
        'group_property_ids': fields.many2many('mrp.property.group.option', 'mrp_production_lot_group_property_rel','production_lot_id', 'group_id', 'Product Property Group Rel '),
        'option_ids': fields.many2many('product.links', 'mrp_production_lot_options_rel','production_lot_id', 'links_id', 'Options'),
    }
    _defaults = {
        'size_x': lambda * a: 0.0,
        'size_y': lambda * a: 0.0,
        'size_z': lambda * a: 0.0,
        'shape': lambda * a: 'other',
        'diameter': lambda * a: 0.0,
        'weight': lambda * a: 0.0,
    }

    def generate_serial(self, cr, uid, ids, context={}):
       radio = 0
       for w in self.browse(cr, uid, ids):
           v = {
                'name' : name_serial(w.size_x, w.size_y, w.size_z, w.weight, w.shape, w.diameter)
                }
           w.write(v)

    def compute_weight(self, cr, uid, id, product_id, size_x, size_y, size_z, shape, density, diameter):
       if product_id:
           product = self.pool.get('product.product').browse(cr, uid, product_id, context='')
           factor1 = product.uom_d_weight.factor
           factor2 = product.uom_id.factor
           factor3 = product.uom_d_size.factor
           factor4 = product.uom_s_size.factor           
           return {'value': compute_w(cr, uid, factor1, factor2, factor3, factor4, size_x, size_y, size_z, shape, density, diameter)}

stock_production_lot()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: