# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2007-2014 ASPerience SARL (<http://www.asperience.fr>).
#    All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from openerp.osv import fields,osv
from openerp.tools.translate import _
import re
import openerp.addons.decimal_precision as dp

class product_label_line(osv.osv):
    _name = "product.label.line"
    _description = 'product.label.line'
    _columns = {
        'code':fields.char('Code', size=16, required=True),
        'name':fields.char('Name', size=51, required=True),
        'sequence': fields.integer('Sequence', required=True),
        'product_id':fields.many2one('product.product', 'Product'),
    }
    _defaults = {
        'sequence': lambda *a : 1,
    }
    _order = 'sequence,code,name'

    def on_change_name(self, cr, uid, id, name):
        if not name:
            return {}
        if len(name) > 51 :
            warning = {'title':'User Alert!','message':'name is too long ( max 51 )'}
            return {'warning':warning}
        return {}
    
product_label_line()

class product_class(osv.osv):
    _name = 'product.class'
    _columns = {
        'code': fields.char('Code', required=True, size=16),
        'name': fields.char('Name', required=True, size=46 ),
        'domain': fields.selection([('product.class','Product type'),('product.class.option','Option type')], 'Type', required=True, size=24)
    }
    _order = 'domain,code,name'
product_class()

class product_type_attributes(osv.osv):
    _name = 'product.type.attributes'
    _description = 'Attributes types'
    _columns = {
        'code': fields.char('Code', required=True, size=16),
        'name': fields.char('Name', required=True, size=256),
        'product_class': fields.many2one('product.class', 'Product Class', ondelete='set null'),
        'domain': fields.selection([('product.template','Template Attribute'),('product.product','Product Attribute'),('stock.move','Stock move'),('stock.production.lot','Stock production lot'),('stock.tracking','Stock tracking')], 'Type', size=24)
    }
    _order = 'domain,code,name'
product_type_attributes()

class product_type_property_option(osv.osv):
    _name = 'product.type.property.option'
    _description = 'GOP or GOO'
    _columns = {
        'code': fields.char('Code', required=True, size=16),
        'name': fields.char('Name', required=True, size=51),
        'domain': fields.selection([('product.category.links','Options'),('mrp.property','Property')], 'Type', required=True, size=24),
        'sequence': fields.integer('Sequence'),
        'code_prod': fields.boolean('Product Code'),
        'execution_filter': fields.boolean('Execution Filter'),
        'position_code': fields.selection([('suffix','Suffix'),('prefix','Prefix')], 'Position Code', required=True, size=24),
        'pre_suffix': fields.char('Separator', size=6),
        'required': fields.selection([('false',_('False')),('true',_('True')),('true_cs',_('True except CS'))], 'Required', size=20),
        'to': fields.selection([('produce',_('Produce')),('sale',_('Sale'))], 'To', size=20),
        'parent_id': fields.many2one('product.type.property.option','Parent' ),
        'product_links_ids': fields.one2many('product.links', 'type_group', 'Product links'),
        'product_links_bound_ids': fields.one2many('product.links', 'bound_type_group', 'Bound Product links'),
        'product_category_property_ids': fields.one2many('product.category.property', 'type_group', 'Product categories'),
        'product_category_links_ids': fields.one2many('product.category.links', 'type_group', 'Product links'),
        'product_category_links_bound_ids': fields.one2many('product.category.links', 'bound_type_group', 'Bound Product links'),
    }

    _defaults = {
        'sequence': lambda *a: 10,
        'position_code': lambda *a: 'suffix',
        'required': lambda *a: 'false',
        'to': lambda *a: 'produce',
    }
    _order = 'domain,sequence,code,name'
product_type_property_option()

class product_property(osv.osv):
    _name = 'product.property'
    _description = 'OCP'
    _columns = {
        'name': fields.char('Description', size=256, required=True),
        'product_id' : fields.many2one('product.product', 'Product', ondelete='cascade'),
        'template_id' : fields.many2one('product.template', 'Product', ondelete='cascade'),
        'property_id': fields.many2one('mrp.property.group', 'Property', required=True),
        'type_group': fields.many2one('product.type.property.option', 'GOP'),
        'used': fields.boolean('Used'),
    }
    _defaults = {
        'used': lambda *a: True,
    }
    _order = 'product_id,template_id,used desc,type_group, name,property_id'
    
    def onchange_property(self, cr, uid, id, property_id):
        res = {'value':{}}
        if not property_id:
            return res
        property_group = self.pool.get('mrp.property.group').browse(cr,uid,property_id)
        name = property_group.name or ''
        if name:
            res['value']['name'] = name
        return res
    
product_property()

class product_links(osv.osv):
    _name = 'product.links'
    
    def check(self, cr, uid, vals, context=None):
        if 'ids' in vals and vals['ids']:
            link = self.browse(cr,uid,vals['ids'])[0]
            if not 'link_type' in vals:
                vals['link_type'] = link.link_type.id
            if not 'type_group' in vals:
                vals['type_group'] = link.type_group.id
        if 'link_type' in vals and 'type_group' in vals :
            if self.pool.get('product.class').browse(cr,uid,vals['link_type']).code in ('option') and not vals['type_group'] :
                raise osv.except_osv(_('Warning !'),_('Please add a Group By. (GOO)'))
        bound_type_group = False
        bound_property = False
        if 'bound_type_group' in vals :
            bound_type_group = vals['bound_type_group']
        if 'bound_property' in vals :
            bound_property = vals['bound_property']
        if not((not bound_type_group and not bound_property) or (bound_type_group and     bound_property)):
            raise osv.except_osv(_('Warning !'),_('Please check bounds'))
         
    def create(self, cr, uid, vals, context=None):
        self.check(cr, uid, vals, context)
        return super(product_links, self).create(cr, uid, vals, context)
    
    def write(self, cr, uid, ids, vals, context=None):
        if not 'ids' in vals:
            vals['ids'] = ids
        self.check(cr, uid, vals, context)
        return super(product_links, self).write(cr, uid, ids, vals, context)

    _columns = {
        'sequence': fields.integer('Sequence'),
        'name': fields.char('Description', size=256, required=True),
        'product_id' : fields.many2one('product.product', 'Product', required=True, ondelete='cascade'),
        'link_type': fields.many2one('product.class', 'Link Type', required=True, ondelete='set null'),
        'type_group': fields.many2one('product.type.property.option', 'GOO'),
        'bound_type_group': fields.many2one('product.type.property.option', 'Bound GOP'),
        'bound_property': fields.many2one('mrp.property', 'Bound Property'),
        'product_base_id': fields.many2one('product.product', 'Product Base', ondelete='cascade'),
        'template_base_id': fields.many2one('product.template', 'Template Base', ondelete='cascade'),
        'product_qty': fields.integer('Quantity', required=True),
        'used': fields.boolean('Used'),
        'default': fields.boolean('Default'),
        'specific': fields.boolean('Specific'),
        'specific_price': fields.float('Specific Price', digits=(16, 5)),
    }
    _defaults = {
        'product_qty': lambda *a: 1,
        'sequence': lambda *a: 1,
        'used': lambda *a: False,
        'specific': lambda *a: False,
        'specific_price': lambda *a: 0,
    }
    _order = 'used desc,type_group,sequence,name'

    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=80):
        if not args:
            args=[]
        if not context:
            context={}
        result = []
        sql = "select distinct A.id from product_links A, \
                product_product B , product_template C , \
                product_product D , product_template E \
                where \
                A.product_id = B.id AND B.product_tmpl_id = C.id AND \
                A.product_base_id = D.id AND D.product_tmpl_id = E.id AND \
                (C.name ilike '%"+name.strip()+"%' OR E.name ilike '%"+name.strip()+"%' OR \
                B.default_code ilike '%"+name.strip()+"%' OR D.default_code ilike '%"+name.strip()+"%' )"
        cr.execute(sql)
        result_tmp = cr.fetchall()
        for id in result_tmp:
            result.append([id[0]])
        return result
    
    def onchange_link(self, cr, uid, id, product_id):
        res = {'value':{}}
        if not product_id:
            return res
        product = self.pool.get('product.product').browse(cr,uid,product_id)
        name = ((product.code and '['+product.code+'] ') or '') +product.name
        if name:
            res['value']['name'] = name
        return res
    
product_links()

class product_product_attribute(osv.osv):
    _name = "product.product.attribute"
    _description = "Product attributes"
    _columns = {
        'sequence': fields.integer('Sequence', required=True),
        'type': fields.many2one('product.type.attributes', 'Type', ondelete='set null'),
        'index' : fields.char('Attribute', size=128, required=True, select=True),
        'value' : fields.text('Value', select=True),
        'product_id': fields.many2one('product.product', 'Product', required=True, ondelete='cascade'),
    }
    _defaults = {
        'sequence': lambda *a: 10,
    }
    _order = "product_id,sequence, id"

    def name_get(self, cr, user, ids, context={}):
        if not len(ids):
            return []
        def _name_get(d):
            name = d.get('type','') + ' ' + d.get('index','') + ' ' + d.get('value','') 
            return (d['id'], name)
        result = map(_name_get, self.read(cr, user, ids, [], context))
        return result

    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=80):
        if not args:
            args=[]
        if not context:
            context={}
        
        name_search = name.split('+')
        sql_search = ''
        sql_header = ''
        result = []
        index = 0
        for data_search in name_search:
            if index  <> 0:
                sql_search += ' AND '
                sql_header += ' , '
            sql_search += " (A"+str(index)+".type ilike '%"+data_search.strip()+"%' OR A"+str(index)+".index ilike '%"+data_search.strip()+"%' OR A"+str(index)+".value ilike '%"+data_search.strip()+"%') "
            sql_header += " product_product_attribute A"+str(index)+" "
            if index  <> 0:
                sql_search += " AND A0.product_id = A"+str(index)+".product_id "
            index += 1 
            
        sql = 'select distinct A0.id from '+sql_header + ' where '+sql_search
        cr.execute(sql)

        ids = filter(None, map(lambda x:x[0], cr.fetchall()))
        for id in ids:
            result.append([id])
        return result

product_product_attribute()

class product_product(osv.osv):
    _inherit ='product.product'
    
    def create(self, cr, uid, vals, context=None):
        id = super(product_product, self).create(cr, uid, vals, context)
        if context and "copy" in context and context["copy"]:
            return id
        categ = self.browse(cr,uid,id).product_tmpl_id.categ_id
        for links in categ.linked_product_ids:
            if links.type == "product.product":
                vals = {
                            'name' : links.name,
                            'product_id' : links.product_id.id,
                            'link_type' : links.link_type.id,
                            'product_base_id' : id,
                            'product_qty': links.product_qty,
                            'type_group' : links.type_group.id,
                            'used' : links.used,
                            'bound_type_group': links.bound_type_group.id,
                            'bound_property': links.bound_property.id,
                            'sequence': links.sequence,
                            'specific': links.specific,
                            'specific_price': links.specific_price,
                }
                self.pool.get('product.links').create(cr,uid,vals)
        for property in categ.property_ids:
            if property.type == "product.product":
                vals = {
                            'name' : property.name,
                            'product_id' : id,
                            'property_id' : property.property_id.id,
                            'type_group' : property.type_group.id,
                            'used' : property.used,
                }
                self.pool.get('product.property').create(cr,uid,vals)        
        return id
    
    def copy(self, cr, uid, id, default={}, context={}):
        #default.update( {'product_variant_ids':False})
        context.update( {'copy':True})
        return super(product_product, self).copy(cr, uid, id, default, context)

    def get_attribut_text(self,cr,uid,ids,context={}):
        result = {}
        for product_ptr in self.browse(cr,uid,ids,context):
            attr_str = ""
            for attr_ptr in product_ptr.product_attribute_ids:
                attr_str += ( attr_ptr.type or "" ) + "\t"
                attr_str += ( attr_ptr.index or "" ) + "\t"
                attr_str += ( attr_ptr.value or "" ) + "\r\n"
            for attr_ptr in product_ptr.product_tmpl_id.template_attributes_ids:
                attr_str += ( attr_ptr.type or "" ) + "\t"
                attr_str += ( attr_ptr.index or "" ) + "\t"
                attr_str += ( attr_ptr.value or "" ) + "\r\n"
            result[product_ptr.id] = attr_str
        return result 

    def _is_configurable(self, cr, uid, ids, name, arg, context=None):
        res = {}
        for product in self.browse(cr, uid, ids):
            res[product.id] = False

            for product_link in product.linked_product_ids:
                if product_link.link_type.code == 'option' and product_link.used:
                    res[product.id] = 'configurable'
                    break
                elif product_link.link_type.code == 'cross_sell' and product_link.used:
                    res[product.id] = 'cross_sell'
            
            if res[product.id] == 'configurable':
                continue
            
            for product_tmpl_link in product.product_tmpl_id.linked_product_tmpl_ids:
                if product_tmpl_link.link_type.code == 'option' and product_tmpl_link.used:
                    res[product.id] = 'configurable'
                    break
                elif product_tmpl_link.link_type.code == 'cross_sell' and product_tmpl_link.used:
                    res[product.id] = 'cross_sell'
            
            if res[product.id] == 'configurable':
                continue
            
            for product_property in product.property_ids:
                if product_property.used:
                    res[product.id] = 'configurable'
                    break
            
            if res[product.id] == 'configurable':
                continue
            
            for product_tmpl_property in product.product_tmpl_id.property_tmpl_ids:
                if product_tmpl_property.used:
                    res[product.id] = 'configurable'
                    break
        return res        

    def del_all_properties(self, cr, uid, ids, context=None):
        tmpl_ids = [i.product_tmpl_id.id for i in self.browse(cr,uid,ids,context)]
        property_obj = self.pool.get('product.property')
        property_ids = property_obj.search(cr,uid,[('product_id','in',ids)],context)
        property_ids.extend(property_obj.search(cr,uid,[('template_id','in',tmpl_ids)],context))
        if property_ids:
            property_obj.unlink(cr,uid,property_ids,context)
        
    def del_all_links(self, cr, uid, ids, context=None):
        tmpl_ids = [i.product_tmpl_id.id for i in self.browse(cr,uid,ids,context)]
        links_obj = self.pool.get('product.links')
        links_ids = links_obj.search(cr,uid,[('product_base_id','in',ids)],context)
        links_ids.extend(links_obj.search(cr,uid,[('template_base_id','in',tmpl_ids)],context))
        if links_ids:
            links_obj.unlink(cr,uid,links_ids,context)

    def reinit_links(self, cr, uid, ids, context=None):
        for product_ptr in self.browse(cr,uid,ids,context):
            product_ptr.del_all_properties()
            product_ptr.del_all_links()
            categ = product_ptr.product_tmpl_id.categ_id
            for property in categ.property_ids:
                if property.type == "product.product":
                    vals = {
                                'name' : property.name,
                                'product_id' : product_ptr.id,
                                'property_id' : property.property_id.id,
                                'type_group' : property.type_group.id,
                                'used' : property.used,
                    }
                    self.pool.get('product.property').create(cr,uid,vals)
                if property.type == "product.template":
                    vals = {
                                'name' : property.name,
                                'template_id' : product_ptr.product_tmpl_id.id,
                                'property_id' : property.property_id.id,
                                'type_group' : property.type_group.id,
                                'used' : property.used,
                    }
                    self.pool.get('product.property').create(cr,uid,vals)
            for links in categ.linked_product_ids:
                if links.type == "product.product":
                    vals = {
                                'name' : links.name,
                                'product_id' : links.product_id.id,
                                'link_type' : links.link_type.id,
                                'product_base_id' : product_ptr.id,
                                'product_qty': links.product_qty,
                                'type_group' : links.type_group.id,
                                'used' : links.used,
                                'bound_type_group': links.bound_type_group.id,
                                'bound_property': links.bound_property.id,
                                'sequence': links.sequence,
                                'specific': links.specific,
                                'specific_price': links.specific_price,
                    }
                    self.pool.get('product.links').create(cr,uid,vals)
                if links.type == "product.template":
                    vals = {
                                'name' : links.name,
                                'product_id' : links.product_id.id,
                                'link_type' : links.link_type.id,
                                'template_base_id' : product_ptr.product_tmpl_id.id,
                                'product_qty': links.product_qty,
                                'type_group' : links.type_group.id,
                                'used' : links.used,
                                'bound_type_group': links.bound_type_group.id,
                                'bound_property': links.bound_property.id,
                                'sequence': links.sequence,
                                'specific': links.specific,
                                'specific_price': links.specific_price,
                    }
                    self.pool.get('product.links').create(cr,uid,vals)

    # TODO: ajouter l'initialisation des attributs template/product lors du changement de type de produit
    _columns = {
        'product_attribute_ids': fields.one2many('product.product.attribute', 'product_id', 'Attributes', copy=True),
        'property_ids': fields.one2many('product.property', 'product_id', 'Property Product', copy=True),
        'linked_product_ids': fields.one2many('product.links', 'product_base_id', 'Linked products product', copy=True),
        'is_configurable': fields.function(_is_configurable, readonly=True, method=True, store=True, type='selection', string='Is Configurable', selection=[(False,_('Not Configurable')),('cross_sell',_('Cross Sell')),('configurable',_('Configurable'))]),
    }
    
    _defaults = {
    }
product_product()

class product_template_attribute(osv.osv):
    _name = "product.template.attribute"
    _description = "Product template attributes"
    _columns = {
        'sequence': fields.integer('Sequence', required=True),
        'type': fields.many2one('product.type.attributes', 'Type', ondelete='set null'),
        'index' : fields.char('Attribute', size=128, required=True),
        'value' : fields.text('Value', select=True),
        'template_id': fields.many2one('product.template', 'Template', required=True, ondelete='cascade'),
    }
    _defaults = {
        'sequence': lambda *a: 10,
    }
    _order = "template_id, sequence, id"

    def name_get(self, cr, user, ids, context={}):
        if not len(ids):
            return []
        def _name_get(d):
            name = d.get('type','') + ' ' + d.get('index','') + ' ' + d.get('value','') 
            return (d['id'], name)
        result = map(_name_get, self.read(cr, user, ids, [], context))
        return result

    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=80):
        if not args:
            args=[]
        if not context:
            context={}
        
        name_search = name.split('+')
        sql_search = ''
        sql_header = ''
        result = []
        index = 0
        for data_search in name_search:
            if index  <> 0:
                sql_search += ' AND '
                sql_header += ' , '
            sql_search += " (A"+str(index)+".type ilike '%"+data_search.strip()+"%' OR A"+str(index)+".index ilike '%"+data_search.strip()+"%' OR A"+str(index)+".value ilike '%"+data_search.strip()+"%') "
            sql_header += " product_template_attribute A"+str(index)+" "
            if index  <> 0:
                sql_search += " AND A0.template_id = A"+str(index)+".template_id "
            index += 1 
            
        sql = 'select distinct A0.id from '+sql_header + ' where '+sql_search
        cr.execute(sql)

        ids = filter(None, map(lambda x:x[0], cr.fetchall()))
        for id in ids:
            result.append([id])
        return result

product_template_attribute()

class product_template(osv.osv):
    _inherit ='product.template'
    
    def create(self, cr, uid, vals, context=None):
        id = super(product_template, self).create(cr, uid, vals, context)
        if context and "copy" in context and context["copy"]:
            return id
        categ = self.browse(cr,uid,id).categ_id
        for links in categ.linked_product_ids:
            if links.type == "product.template":
                vals = {
                            'name' : links.name,
                            'product_id' : links.product_id.id,
                            'link_type' : links.link_type.id,
                            'template_base_id' : id,
                            'product_qty': links.product_qty,
                            'type_group' : links.type_group.id,
                            'used' : links.used,
                            'bound_type_group': links.bound_type_group.id,
                            'bound_property': links.bound_property.id,
                            'sequence': links.sequence,
                            'specific': links.specific,
                            'specific_price': links.specific_price,
                }
                self.pool.get('product.links').create(cr,uid,vals)
        for property in categ.property_ids:
            if property.type == "product.template":
                vals = {
                            'name' : property.name,
                            'template_id' : id,
                            'property_id' : property.property_id.id,
                            'type_group' : property.type_group.id,
                            'used' : property.used,
                }
                self.pool.get('product.property').create(cr,uid,vals)
        return id

    def copy(self, cr, uid, id, default={}, context={}):
        context.update( {'copy':True})
        return super(product_template, self).copy(cr, uid, id, default, context)

    def _is_configurable_tmpl(self, cr, uid, ids, name, arg, context=None):
        res = {}
        for template in self.browse(cr, uid, ids):
            is_configurable_tmpl = False
            for product in template.product_variant_ids:
                is_configurable = product.is_configurable
                print "is_configurable:",is_configurable
                #configurable si au moins 1 variante configurable
                if is_configurable:
                    is_configurable_tmpl == is_configurable
                #configurable prévaut sur cross_sell
                if is_configurable_tmpl == 'configurable':
                    break
            res[template.id] = is_configurable_tmpl
        return res        
            
    _columns = {
        'product_class': fields.many2one('product.class', 'Product Class', ondelete='set null'),
        'is_configurable_tmpl': fields.function(_is_configurable_tmpl, readonly=True, method=True, store=True, type='selection', string='Is Configurable', selection=[(False,_('Not Configurable')),('cross_sell',_('Cross Sell')),('configurable',_('Configurable'))]),
        'template_attributes_ids': fields.one2many('product.template.attribute', 'template_id', 'Template Attributes', copy=True),
        'property_tmpl_ids': fields.one2many('product.property', 'template_id', 'Property Template', copy=True),
        'linked_product_tmpl_ids': fields.one2many('product.links', 'template_base_id', 'Linked products template', copy=True),
    }
    
    _order = 'product_class,tmpl_code,name'
    
product_template()

class product_category_property(osv.osv):
    _name = 'product.category.property'
    _description = 'OCP'
    
    def onchange_property(self, cr, uid, id, property_id):
        res = {'value':{}}
        if not property_id:
            return res
        property_group = self.pool.get('mrp.property.group').browse(cr,uid,property_id)
        name = property_group.name or ''
        if name:
            res['value']['name'] = name
        return res

    def create(self, cr, uid, vals, context=None):
        id = super(product_category_property, self).create(cr, uid, vals, context)
        property = self.browse(cr,uid,id)
        template_ids = self.pool.get('product.template').search(cr,uid,[('categ_id','=',property.categ_id.id)])
        if property.type == "product.product":
            product_ids = self.pool.get('product.product').search(cr,uid,[('product_tmpl_id','in',template_ids)])
            for product_id in product_ids:
                tmpl = self.pool.get('product.product').read(cr,uid,product_id,['product_tmpl_id'])['product_tmpl_id'][0]
                vals = {
                            'name' : property.name,
                            'product_id' : product_id,
                            'property_id' : property.property_id.id,
                            'type_group' : property.type_group.id,
                            'used' : property.used,
                }
                filter = [ ('product_id','=',vals['product_id']),
                           ('property_id','=',vals['property_id']),
                           ('type_group','=',vals['type_group']),
                           ('used','=',vals['used']),
                ]
                if not self.pool.get('product.property').search(cr,uid,filter) :
                    filter = [ ('template_id','=',tmpl),
                               ('property_id','=',vals['property_id']),
                               ('type_group','=',vals['type_group']),
                               ('used','=',vals['used']),
                    ]
                    if not self.pool.get('product.property').search(cr,uid,filter) :
                        self.pool.get('product.property').create(cr,uid,vals)
        elif property.type == "product.template":
            for template_id in template_ids:
                vals = {
                            'name' : property.name,
                            'template_id' : template_id,
                            'property_id' : property.property_id.id,
                            'type_group' : property.type_group.id,
                            'used' : property.used,
                }
                filter = [ ('template_id','=',vals['template_id']),
                           ('property_id','=',vals['property_id']),
                           ('type_group','=',vals['type_group']),
                           ('used','=',vals['used']),
                   ]
                if not self.pool.get('product.property').search(cr,uid,filter) :
                    self.pool.get('product.property').create(cr,uid,vals)
        
        return id
    
    _columns = {
        'name': fields.char('Description', size=256, required=True),
        'categ_id' : fields.many2one('product.category', 'Category', ondelete='cascade'),
        'property_id': fields.many2one('mrp.property.group', 'Property Group', required=True),
        'type': fields.selection([('product.product', 'Product'), ('product.template', 'Template')], 'Type', size=32, required=True),
        'type_group': fields.many2one('product.type.property.option', 'GOP'),
        'used': fields.boolean('Used'),
    }
    _order = 'type,type_group,name'
    _defaults = {
        'used': lambda *a: True,
    }
product_category_property()

class product_category_links(osv.osv):
    _name = 'product.category.links'
    _columns = {
        'sequence': fields.integer('Sequence'),
        'name': fields.char('Description', size=256, required=True),
        'product_id' : fields.many2one('product.product', 'Product', required=True, ondelete='cascade'),
        'link_type': fields.many2one('product.class', 'Link Type', required=True, ondelete='set null'),
        'category_base_id': fields.many2one('product.category', 'Category', required=True, ondelete='cascade'),
        'type_group': fields.many2one('product.type.property.option', 'GOO'),
        'bound_type_group': fields.many2one('product.type.property.option', 'Bound GOP'),
        'bound_property': fields.many2one('mrp.property', 'Bound Property'),
        'product_qty': fields.integer('Quantity', required=True),
        'type': fields.selection([('product.product', _('Product')), ('product.template', _('Template'))], 'Type', size=32, required=True),
        'used': fields.boolean('Used'),
        'specific': fields.boolean('Specific'),
        'specific_price': fields.float('Specific Price', digits=(16, 5)),
    }
    _order = 'type,used desc,type_group,sequence,name'
    _defaults = {
        'sequence': lambda *a: 1,
        'product_qty': lambda *a: 1,
        'used': lambda *a: False,
        'specific': lambda *a: False,
        'specific_price': lambda *a: 0,
    }

    def check(self, cr, uid, vals, context=None):
        if 'link_type' in vals and 'type_group' in vals :
            if self.pool.get('product.class').browse(cr,uid,vals['link_type']).code in ('option') and not vals['type_group'] :
                raise osv.except_osv(_('Warning !'),_('Please add a Group By. (GOO)'))

    def create(self, cr, uid, vals, context=None):
        print "product.category.links create vals:",vals
        self.check(cr, uid, vals, context)
        id = super(product_category_links, self).create(cr, uid, vals, context)
        links = self.browse(cr,uid,id)
        template_ids = self.pool.get('product.template').search(cr,uid,[('categ_id','=',links.category_base_id.id)])
        if links.type == "product.product":
            product_ids = self.pool.get('product.product').search(cr,uid,[('product_tmpl_id','in',template_ids)])
            for product_id in product_ids:
                tmpl = self.pool.get('product.product').read(cr,uid,product_id,['product_tmpl_id'])['product_tmpl_id'][0]
                vals = {
                            'name' : links.name,
                            'product_id' : links.product_id.id,
                            'link_type' : links.link_type.id,
                            'product_base_id' : product_id,
                            'product_qty': links.product_qty,
                            'type_group' : links.type_group.id,
                            'used' : links.used,
                            'bound_type_group': links.bound_type_group.id,
                            'bound_property': links.bound_property.id,
                            'sequence': links.sequence,
                            'specific': links.specific,
                            'specific_price': links.specific_price,
                }
                filter = [ ('product_id','=',vals['product_id']),
                           ('link_type','=',vals['link_type']),
                           ('product_base_id','=',vals['product_base_id']),
                           ('product_qty','=',vals['product_qty']),
                           ('type_group','=',vals['type_group']),
                           ('bound_type_group','=',vals['bound_type_group']),
                           ('bound_property','=',vals['bound_property'])
                ]
                if not self.pool.get('product.links').search(cr,uid,filter) :
                    filter = [ ('product_id','=',vals['product_id']),
                               ('link_type','=',vals['link_type']),
                               ('template_base_id','=',tmpl),
                               ('product_qty','=',vals['product_qty']),
                               ('type_group','=',vals['type_group']),
                               ('bound_type_group','=',vals['bound_type_group']),
                               ('bound_property','=',vals['bound_property']),
                       ]
                    if not self.pool.get('product.links').search(cr,uid,filter) :
                        self.pool.get('product.links').create(cr,uid,vals)
        elif links.type == "product.template":
            for template_id in template_ids:
                vals = {
                            'name' : links.name,
                            'product_id' : links.product_id.id,
                            'link_type' : links.link_type.id,
                            'template_base_id' : template_id,
                            'product_qty': links.product_qty,
                            'type_group' : links.type_group.id,
                            'used' : links.used,
                            'bound_type_group': links.bound_type_group.id,
                            'bound_property': links.bound_property.id,
                            'sequence': links.sequence,
                            'specific': links.specific,
                            'specific_price': links.specific_price,
                }
                filter = [ ('product_id','=',vals['product_id']),
                           ('link_type','=',vals['link_type']),
                           ('template_base_id','=',template_id),
                           ('product_qty','=',vals['product_qty']),
                           ('type_group','=',vals['type_group']),
                           ('bound_type_group','=',vals['bound_type_group']),
                           ('bound_property','=',vals['bound_property']),
                   ]
                if not self.pool.get('product.links').search(cr,uid,filter) :
                    self.pool.get('product.links').create(cr,uid,vals)
        
        return id

    def write(self, cr, uid, vals, context=None):
        print "product.category.links write vals:",vals
        self.check(cr, uid, vals, context)
        id = super(product_category_links, self).write(cr, uid, vals, context)
    
    def onchange_link(self, cr, uid, id, product_id):
        res = {'value':{}}
        if not product_id:
            return res
        product = self.pool.get('product.product').browse(cr,uid,product_id)
        #name = ((product.code and '['+product.code+'] ') or '') +product.name
        name = product.complete_name
        if name:
            res['value']['name'] = name
        return res
product_category_links()

class product_category(osv.osv):
    _inherit = 'product.category'
    _columns = {
        'description': fields.text('Description'),
        'property_ids': fields.one2many('product.category.property', 'categ_id', 'Property', copy=True),
        'linked_product_ids': fields.one2many('product.category.links', 'category_base_id', 'Linked products', copy=True),
    }
    _defaults = {
    }
product_category()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: