# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2007-2014 ASPerience SARL (<http://www.asperience.fr>).
#    All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from openerp.osv import fields, osv
import openerp.addons.decimal_precision as dp
import re

class product_template(osv.osv):
    _inherit ='product.template'
    
    def name_get(self, cr, uid, ids, context=None):
        if not ids:
            return []
        res = []
        if isinstance(ids,int):
            ids = [ids]
        for record in self.read(cr, uid, ids, ['name','tmpl_code'], context):
            if record['tmpl_code']:
                name = "[%s] %s" % (record['tmpl_code'],record['name'])
            else:
                name = record['name']
            res.append((record['id'], name))
        return res
    
    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=80):
        if not args:
            args=[]
        if not context:
            context={}
        
        if name:
            ids = self.search(cr, user, [('tmpl_code','=',name)]+ args, limit=limit, context=context)
            if not ids:
                ids += self.search(cr, user, [('name',operator,name)]+ args, limit=limit, context=context)
                if not ids:
                    ids = self.search(cr, user, [('tmpl_code','ilike',name)]+ args, limit=limit, context=context)
            if not ids:
                code = re.compile(r'\[(.*)\]').search(name)
                product_code = ""
                if code:
                    product_code = code.group()
                    name = name.replace(product_code,"")
                    product_code = product_code[1:-1]
                    ids = self.search(cr, user, [('tmpl_code','=',name)]+ args, limit=limit, context=context)
                    
        else:
            ids = self.search(cr, user, args, limit=limit, context=context)
        result = self.name_get(cr, user, ids, context)
        return result
    
    def transform_into_simple_product(self, cr, uid, ids, context=False):
        product_obj = self.pool.get('product.product')
        for template_ptr in self.browse(cr,uid,ids):
            if len(template_ptr.product_variant_ids) <= 1: 
                break
            for variant_ptr in template_ptr.product_variant_ids[1:]:
                default = {
                }
                product_tmpl_id = self.copy(cr, uid, template_ptr.id, default, context)
                vals = {
                    'product_tmpl_id': product_tmpl_id,
                }
                product_obj.write(cr,uid,variant_ptr.id,vals,context)
        return True

    def copy(self, cr, uid, id, default={}, context={}):
        default.update( {'product_variant_ids':False})
        return super(product_template, self).copy(cr, uid, id, default, context)


    def _is_product_variant(self, cr, uid, ids, name, arg, context=None):
        res = {}
        for template in self.browse(cr, uid, ids):
            if len(template.product_variant_ids)>1 :
                res[template.id] = True
            else:
                res[template.id] = False
        return res

    _columns = {
        'tmpl_code' : fields.char('Template Reference', size=64),
        'is_product_variant': fields.function( _is_product_variant, type='boolean', string='Only one product variant', store=True, readonly=True),
    }
    
    _defaults = {
    }
    
product_template()

class product_product(osv.osv):
    _inherit ='product.product'
        
    def _product_list_price_base_get(self, cr, uid, ids, name, arg, context={}):
        return self.price_get(cr, uid, ids, 'list_price', context={})

    def _get_partner_code_name(self, cr, uid, ids, product, partner_id, context={}):
        try:
            manufacturer_name = product.manufacturer
        except:
            manufacturer_name = ""
        for supinfo in product.seller_ids:
            if supinfo.name.id == partner_id:
                return {'code': supinfo.product_code, 'name': supinfo.product_name, 'manufacturer':manufacturer_name}

        product_code = product.tmpl_code or ''
        if product.default_code:
            if product_code:
                product_code = product_code + "/" 
            product_code = product_code + product.default_code
        product_name = product.name
        for attribute in product.attribute_value_ids:
            product_name = product_name + " " + attribute.name
            
        return {'code' : product_code, 'name' : product_name, 'manufacturer':manufacturer_name}

    def _product_code(self, cr, uid, ids, name, arg, context=None):
        res = {}
        if context is None:
            context = {}
        for p in self.browse(cr, uid, ids, context=context):
            data = self._get_partner_code_name(cr, uid, [], p, context.get('partner_id', None), context=context)
            product_code = ""
            manufacturer_name = 'manufacturer' in data and data['manufacturer'] or ''
            if manufacturer_name:
                product_code = manufacturer_name + "/"
            if 'code' in  data and data['code']:
                if product_code:
                    product_code = product_code + "/" 
                product_code = product_code + data['code']
            res[p.id] = product_code

        return res
    
    def _product_partner_ref(self, cr, uid, ids, name, arg, context={}):
        res = {}
        if context is None:
            context = {}
        for p in self.browse(cr, uid, ids, context):
            data = self._get_partner_code_name(cr, uid, [], p, context.get('partner_id', None), context)
            partner_ref = ""
            if data['manufacturer']:
                partner_ref += data['manufacturer']

            if data['code']:
                if partner_ref != "":
                    partner_ref += "/"
                partner_ref += data['code']

            res[p.id] = ( (partner_ref != "") and ('['+partner_ref+'] ') or '') + (data['name'] or '')
        return res

    def name_get(self, cr, user, ids, context={}):
        if not ids:
            return []
        if isinstance(ids,int):
            ids = [ids]
        if not len(ids):
            return []
        def _name_get(d):
            name = self._product_partner_ref(cr, user, [d['id']], '', '', context)[d['id']]
            return (d['id'], name)
        result = map(_name_get, self.read(cr, user, ids, ['id'], context))

        return result
        
    def name_search(self, cr, user, name='', args=None, operator='ilike', context=None, limit=80):
        if not args:
            args=[]
        if not context:
            context={}
        
        if name:
            ids = self.search(cr, user, [('name','=',name)]+ args, limit=limit, context=context)
            if not len(ids):
                ids = self.search(cr, user, [('default_code','=',name)]+ args, limit=limit, context=context)
            if not len(ids):
                ids = self.search(cr, user, [('tmpl_code','=',name)]+ args, limit=limit, context=context)
            if not len(ids):
                ids = self.search(cr, user, [('ean13','=',name)]+ args, limit=limit, context=context)
            if not len(ids):
                ids = self.search(cr, user, [('name',operator,name)]+ args, limit=limit, context=context)
                ids += self.search(cr, user, [('default_code',operator,name)]+ args, limit=limit, context=context)
                ids += self.search(cr, user, [('tmpl_code',operator,name)]+ args, limit=limit, context=context)
                ids += self.search(cr, user, [('ean13',operator,name)]+ args, limit=limit, context=context)
            if not len(ids):
                code = re.compile(r'\[(.*)\]').search(name)
                product_code = ""
                if code:
                    product_code = code.group()
                    name = name.replace(product_code,"")
                    product_code = product_code[1:-1]
                    ids = self.search(cr, user, [('default_code','=',product_code)]+ args, limit=limit, context=context)
                    if not len(ids):
                        ids = self.search(cr, user, [('tmpl_code','=',product_code)]+ args, limit=limit, context=context)
                    if not len(ids):
                        ids = self.search(cr, user, [('name','=',product_code)]+ args, limit=limit, context=context)
                    if not len(ids):
                        ids = self.search(cr, user, [('ean13','=',product_code)]+ args, limit=limit, context=context)
                    if not len(ids):
                        ids = self.search(cr, user, [('name',operator,name)]+ args, limit=limit, context=context)
                        ids += self.search(cr, user, [('default_code',operator,name)]+ args, limit=limit, context=context)
                        ids += self.search(cr, user, [('tmpl_code',operator,name)]+ args, limit=limit, context=context)
                        ids += self.search(cr, user, [('ean13',operator,name)]+ args, limit=limit, context=context)
        else:
            ids = self.search(cr, user, args, limit=limit, context=context)
        result = self.name_get(cr, user, ids, context)
        return result

    _columns = {
        'code': fields.function(_product_code, type='char', string='Internal Reference'),
        'partner_ref' : fields.function(_product_partner_ref, type='char', string='Customer ref'),
        'standard_price': fields.property(type = 'float', digits_compute=dp.get_precision('Product Price'), 
                                          help="Cost price of the product used for standard stock valuation in accounting and used as a base price on purchase orders.", 
                                          groups="base.group_user", string="Cost Price"),
        'lst_price_variant': fields.function(_product_list_price_base_get, method=True, type='float', string='List price variant'),
    }
    
product_product()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: